# MT::Plugins::Redland::CMS::Topic
# -----------------
# $Revision: 1.6 $
# $Date: 2005/04/06 15:09:45 $
# ---------------------------------------------------------

package MT::Plugins::Redland::CMS::Topic;

use strict;
use warnings;

use Carp qw(confess);
use IO::All;
use Data::Dumper;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data, $debug);
BEGIN {
	no warnings 'numeric';
	$NAME			= "Redland Entry Topic";
	$VERSION		= do { my @REV = split(/\./, (qw$Revision: 1.6 $)[1]); sprintf("%0.3f", $REV[0] + ($REV[1]/1000)) };
	$debug			= 0;
	$plugin_data	= {
		name		=> "${NAME}, v${VERSION}",
		description => 'Adds foaf:topic statements to an Entry',
		doc_link	=> 'http://kasei.us/code/mtsemweb/'
	};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
	
	my $cfg		= MT::ConfigMgr->instance;
	my $cms		= File::Spec->catfile( $cfg->PluginPath, 'redland_cms.pl' );
	require $cms;
}

######################################################################

my $cfg		= MT::ConfigMgr->instance;
if ($cfg->ObjectDriver eq 'Redland') {
	MT::Plugins::Redland::CMS->install_cms_header( $NAME, sub {
		my $app		= shift;
		my $q		= $app->{query};
		my $mode	= $q->param('__mode') || '';
		my $type	= $q->param('_type') || '';
		if ($type eq 'entry' and $mode eq 'view') {
			my $cfg		= MT::ConfigMgr->instance;
			my $path	= $cfg->StaticWebPath || $app->path;
			return qq(<script type="text/javascript" src="${path}easytoggle3.js"></script>);
		}
	} );
	
	MT::Plugins::Redland::CMS->install_cms_box( $NAME, sub {
		my $app		= shift;
		my $q		= $app->{query};
		my $mode	= $q->param('__mode') || '';
		my $type	= $q->param('_type') || '';
		if ($type eq 'entry' and $mode eq 'view') {
			my @first;
			my @rest;
			my $count	= 1;
			our %topics;
			while (my($k,$v) = each(%topics)) {
				my $body	= ref($v) ? $v->( $app ) : $v;
				push(@first, qq(<li><a class="toggle" title="${k}" href="#${k}">${k}</a></li>));
				push(@rest, qq(<div id="${k}">${body}</div>));
			} continue { $count++ }
			
			my $body	= qq(<div><label>Entry Topic</label></div>\n)
						. qq(<div id="toggle"><ul class="toggle">)
						. join("\n", @first)
						. qq(</ul></div>\n)
						. qq(<div id="togglebox">) . join("\n", @rest) . qq(</div>\n);
			return $body;
		}
	} );
}

######################################################################

sub install_topic {
	my $class	= shift;
	my $name	= shift;
	my $body	= shift;
	our %topics;
	$topics{ $name }	= $body;
}

sub MT::Entry::rdf_topic_of_type {
	my $entry	= shift;
	my $type	= shift;
	my @topics	= $entry->rdf_topics;
	my $model	= $entry->driver->model;
	my $p_type	= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
	foreach my $topic (@topics) {
		warn "topic: " . $topic->as_string if ($debug);
		my $st	= RDF::Redland::Statement->new( $topic, $p_type, $type );
		warn $st->as_string if ($debug);
		if ($model->contains_statement( $st )) {
			return $topic;
		}
	}
	
}

sub MT::Entry::rdf_topics {
	my $entry	= shift;
	my $model	= $entry->driver->model;
	my $node	= $entry->get_rdf_node;
	my $p_topic	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/topic' );
	my @topics	= $model->targets( $node, $p_topic );
	return @topics;
}

sub MT::Entry::remove_rdf_topics_of_type {
	my $entry	= shift;
	my $type	= shift;
	my $node	= $entry->get_rdf_node;
	my $model	= $entry->driver->model;
	my @topics	= $entry->rdf_topics();
	my $p_type	= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
	my $p_topic	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/topic' );
	
	foreach my $topic (@topics) {
		my @types	= $model->targets( $topic, $p_type );
		foreach my $topictype (@types) {
			if ($type->equals( $topictype )) {
				my $st		= RDF::Redland::Statement->new( $node, $p_topic, $topic );
				$model->remove_statement( $st );
				last;
			}
		}
	}
}


1;

__END__

=head1 REVISION HISTORY

 $Log: redland_cms_topic.pl,v $
 Revision 1.6  2005/04/06 15:09:45  greg
 - bumped version number to 0.005
 - updated email address in AUTHOR POD section

 Revision 1.5  2005/04/05 05:35:03  greg
 - updated docs
 - now only addes CMS hooks if Redland is the ObjectDriver

 Revision 1.4  2005/04/04 10:12:45  greg
 - $VERSION now based on CVS revision

 Revision 1.3  2005/04/04 07:46:37  greg
 - moved topic_of_type() and topics() into MT::Entry namespace as
   rdf_topic_of_type() and rdf_topics()

 Revision 1.2  2005/04/03 22:19:35  greg
 - topic CMS hooks now only run if on the edit entry page
 - added a foaf:Document topic plugin

 Revision 1.1  2005/04/03 01:07:41  greg
 - added redland CMS Topic plugin
 - moved redland_cms_book.pl to redland_cms_topic_book.pl, now uses Topic API
 - added redland_cms_topic_event.pl for Event topics


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

=over 4

=item * Make Topic tag selected if there's an existing topic

=back

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <greg@evilfunhouse.com>

=cut
